SID_NONE              = 0x00
SID_TIME              = 0x01
SID_LOCATION          = 0x02
SID_PRESSURE          = 0x03
SID_BATTERY           = 0x04
SID_PHYSICAL_LINK     = 0x05
SID_ACCELERATION      = 0x06
SID_TEMPERATURE       = 0x07
SID_HUMIDITY          = 0x08
SID_MAGNETIC_FIELD    = 0x09
SID_AMBIENT_LIGHT     = 0x0A
SID_GRAVITY           = 0x0B
SID_ANGULAR_VELOCITY  = 0x0C
SID_PROXIMITY         = 0x0E
SID_INFORMATION       = 0x0F
SID_RECEIVED          = 0x10
SID_POWER_CONSUMPTION = 0x11
SID_POWER_PRODUCTION  = 0x12
SID_PROCESSOR         = 0x13
SID_RAM               = 0x14
SID_NVM               = 0x15
SID_TANK              = 0x16
SID_FUEL              = 0x17
SID_CUSTOM            = 0xff